
static char    *SccsId = "@(#)header.c 1.1 5/25/92 CNM-US; C source";

/************************************************************************

Copyright (c) 1992 Juan D. Martin
Copyright (c) 1992 Centro Nacional de Microelectronica
Copyright (c) 1992 Universidad de Sevilla

Permission to use, copy, modify, distribute, and sell this software and
its documentation for any purpose is hereby granted without fee,
provided that (i) the above copyright notices and this permission
notice appear in all copies of the software and related documentation,
and (ii) the names of Centro Nacional de Microelectronica and
Universidad de Sevilla may not be used in any advertising or publicity
relating to the software without the specific, prior written
permission.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL THE AUTHOR, CENTRO NACIONAL DE MICROELECTRONICA OR
UNIVERSIDAD DE SEVILLA BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT
OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

************************************************************************/
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xproto.h>
#include "xbdfdump.h"

static char    *warning[] =
{
    "COMMENT  ",
    "COMMENT  WARNING:  This bdf file was generated from an X server using",
    "COMMENT  xbdfdump.  The resulting font is subject to the same copyright,",
    "COMMENT  license, and trademark restrictions as the original font.  The",
    "COMMENT  author and distributors of xbdfdump disclaim all liability for",
    "COMMENT  misuse of the program or its output.",
    "COMMENT  ",
    NULL
};

static int      err_handler(display, e)
Display        *display;
XErrorEvent    *e;
{
    if (e->request_code != X_GetAtomName)
    {
	XmuPrintDefaultErrorMessage(display, e, stderr);
	exit(1);
    }
    return 0;
}

void            print_header()
{
    int             i,
                    j;
    char           *s = (char *) 0;
    char          **w;
    char           *propname;
    char           *fontProperty;
    int             nprops;
    int             bdfprops;
    XFontProp      *properties;
    XCharStruct    *chars;
    Bool            hasdefchar;
    unsigned        pointSize = 0;
    unsigned        Resolution = 0;
    unsigned        xRes = 0;
    unsigned        yRes = 0;
    unsigned        pixelSize = 0;
    unsigned        nchars = 0;
    unsigned long   value;
    int             (*oldhandler) ();

    oldhandler = XSetErrorHandler(err_handler);
    properties = theFontStruct->properties;
    nprops = theFontStruct->n_properties;
    bdfprops = nprops + 3;

    /* Check font name. */
    if (XGetFontProperty(theFontStruct, XA_FONT, &value))
    {
	bdfprops--;		/* FONT does not count for properties in bdf
				 * files. */
	s = XGetAtomName(theDisplay, (Atom) value);
	if (!s)
	{
	    fprintf(stderr,
		    "Warning: font %s , file %s: Bad FONT property data.\n",
		    theFontName, theFileName);
	    fontProperty = theFontName;
	}
	else
	{
	    fontProperty = s;
	}
    }
    else
    {
	fprintf(stderr, "Warning: font %s , file %s: Missing FONT property.\n",
		theFontName, theFileName);
	fontProperty = theFontName;
    }

    /*
     * Search for various prperties related to resolution pixel size, etc.
     * Then figure out the SIZE line for BDF file
     */
    if (XGetFontProperty(theFontStruct, Xa_Pixel_Size, &value))
    {
	pixelSize = value;
    }
    if (XGetFontProperty(theFontStruct, Xa_Resolution_X, &value))
    {
	xRes = value;
    }
    if (XGetFontProperty(theFontStruct, Xa_Resolution_Y, &value))
    {
	yRes = value;
    }
    if (XGetFontProperty(theFontStruct, XA_POINT_SIZE, &value))
    {
	pointSize = (value + 5) / 10;
    }
    if (XGetFontProperty(theFontStruct, XA_RESOLUTION, &value))
    {
	Resolution = value;
	bdfprops--;		/* Resolution does not count in bdf
				 * properties. */
    }
    /* See what we have and try to figure out what we do not have. */
    if (!xRes && Resolution)
    {
	xRes = (Resolution * 7227 + 5000) / 10000;
    }
    else if (!xRes)
    {
	fprintf(stderr, "Warning: font %s , file %s: Faking RESOLUTION_X.\n",
		theFontName, theFileName);
	xRes = 72;
    }
    if (!yRes && Resolution)
    {
	yRes = (Resolution * 7227 + 5000) / 10000;
    }
    else if (!yRes)
    {
	fprintf(stderr, "Warning: font %s , file %s: Faking RESOLUTION_Y.\n",
		theFontName, theFileName);
	xRes = 72;
    }
    if (pixelSize)
    {
	pointSize = pixelSize;
    }
    else if (!pointSize)
    {
	fprintf(stderr, "Warning: font %s , file %s: Faking POINT_SIZE.\n",
		theFontName, theFileName);
	pointSize = ((theFontStruct->ascent + theFontStruct->descent) * 72) /
	    yRes;
    }
    /* Count the number of charaters. */
    chars = theFontStruct->per_char;
    if (fontIsTwoByte)
    {
	charsInFont = (fontMax1 - fontMin1 + 1) * (fontMax2 - fontMin2 + 1);
    }
    else
    {
	charsInFont = fontMax2 - fontMin2 + 1;
    }
    if (chars)
    {
	for (i = 0; i < charsInFont; i++)
	{
	    if (chars[i].width || (chars[i].lbearing != chars[i].rbearing))
		nchars++;
	}
    }
    else
    {
	nchars = charsInFont;
    }
    /* Check default char. */
    if (fontIsTwoByte)
    {
	unsigned        b1,
	                b2,
	                d,
	                i;

	b1 = (theFontStruct->default_char >> 8);
	b2 = (theFontStruct->default_char & 0xFF);
	if (b1 < fontMin1 || b1 > fontMax1 || b2 < fontMin2 || b2 > fontMax2)
	{
	    hasdefchar = False;
	}
	else if (!chars)
	{
	    hasdefchar = True;
	}
	else
	{
	    d = fontMax2 - fontMin2 + 1;
	    i = d * (b1 - fontMin1) + b2 - fontMin2;
	    if (chars[i].width || (chars[i].lbearing != chars[i].rbearing))
	    {
		hasdefchar = True;
	    }
	    else
	    {
		hasdefchar = False;
	    }
	}
    }
    else
    {
	if (theFontStruct->default_char < fontMin2 ||
	    theFontStruct->default_char > fontMax2)
	{
	    hasdefchar = False;
	}
	else if (!chars)
	{
	    hasdefchar = True;
	}
	else
	{
	    i = theFontStruct->default_char - fontMin2;
	    if (chars[i].width || (chars[i].lbearing != chars[i].rbearing))
	    {
		hasdefchar = True;
	    }
	    else
	    {
		hasdefchar = False;
	    }
	}
    }
    if (!hasdefchar)
    {
	bdfprops--;
    }
    /* Print initial stuff */
    fputs("STARTFONT 2.1\n", theFile);
    w = warning;
    while (*w)
    {
	fputs(*w++, theFile);
	putc('\n', theFile);
    }
    fontWidth = theFontStruct->max_bounds.rbearing -
	theFontStruct->min_bounds.lbearing;
    fontHeight = theFontStruct->max_bounds.ascent +
	theFontStruct->max_bounds.descent;
    fprintf(theFile, "FONT %s\n", fontProperty);
    fprintf(theFile, "SIZE %u %u %u\n", pointSize, xRes, yRes);
    fprintf(theFile, "FONTBOUNDINGBOX %d %d %d %d\n", fontWidth, fontHeight,
	    theFontStruct->min_bounds.lbearing,
	    -theFontStruct->max_bounds.descent);
    fprintf(theFile, "STARTPROPERTIES %d\n", bdfprops);
    for (i = 0; i < nprops; i++)
    {
	/* Try to find known things. */
	switch (properties[i].name)
	{
	case XA_MIN_SPACE:
	    fprintf(theFile, "MIN_SPACE %u\n", properties[i].card32);
	    break;
	case XA_NORM_SPACE:
	    fprintf(theFile, "NORM_SPACE %u\n", properties[i].card32);
	    break;
	case XA_MAX_SPACE:
	    fprintf(theFile, "MAX_SPACE %u\n", properties[i].card32);
	    break;
	case XA_END_SPACE:
	    fprintf(theFile, "END_SPACE %u\n", properties[i].card32);
	    break;
	case XA_SUPERSCRIPT_X:
	    fprintf(theFile, "SUPERSCRIPT_X %d\n", properties[i].card32);
	    break;
	case XA_SUPERSCRIPT_Y:
	    fprintf(theFile, "SUPERSCRIPT_Y %d\n", properties[i].card32);
	    break;
	case XA_SUBSCRIPT_X:
	    fprintf(theFile, "SUBSCRIPT_X %d\n", properties[i].card32);
	    break;
	case XA_SUBSCRIPT_Y:
	    fprintf(theFile, "SUBSCRIPT_Y %d\n", properties[i].card32);
	    break;
	case XA_UNDERLINE_POSITION:
	    fprintf(theFile, "UNDERLINE_POSITION %d\n", properties[i].card32);
	    break;
	case XA_UNDERLINE_THICKNESS:
	    fprintf(theFile, "UNDERLINE_THICKNESS %d\n", properties[i].card32);
	    break;
	case XA_STRIKEOUT_ASCENT:
	    fprintf(theFile, "STRIKEOUT_ASCENT %d\n", properties[i].card32);
	    break;
	case XA_STRIKEOUT_DESCENT:
	    fprintf(theFile, "STRIKEOUT_DESCENT %d\n", properties[i].card32);
	    break;
	case XA_ITALIC_ANGLE:
	    fprintf(theFile, "ITALIC_ANGLE %d\n", properties[i].card32);
	    break;
	case XA_X_HEIGHT:
	    fprintf(theFile, "X_HEIGHT %d\n", properties[i].card32);
	    break;
	case XA_QUAD_WIDTH:
	    fprintf(theFile, "QUAD_WIDTH %d\n", properties[i].card32);
	    break;
	case XA_WEIGHT:
	    fprintf(theFile, "WEIGHT %u\n", properties[i].card32);
	    break;
	case XA_POINT_SIZE:
	    fprintf(theFile, "POINT_SIZE %u\n", properties[i].card32);
	    break;
	case XA_RESOLUTION:
	case XA_FONT:
	    break;
	default:
	    print_property(properties[i].name, properties[i].card32);
	}
    }
    if (hasdefchar)
    {
	fprintf(theFile, "DEFAULT_CHAR %u\n", theFontStruct->default_char);
    }
    fontPointSize = pointSize;
    fontResolutionY = yRes;
    fprintf(theFile, "FONT_ASCENT %d\n", theFontStruct->ascent);
    fprintf(theFile, "FONT_DESCENT %d\n", theFontStruct->descent);
    fprintf(theFile, "ENDPROPERTIES\n");
    fprintf(theFile, "CHARS %u\n", nchars);
    /* Free the font name if we got it from an Atom. */
    if (s)
    {
	XFree(s);
    }
    XSetErrorHandler(oldhandler);
}
