
static char    *SccsId = "@(#)main.c 1.1 5/25/92 CNM-US; C source";

/************************************************************************

Copyright (c) 1992 Juan D. Martin
Copyright (c) 1992 Centro Nacional de Microelectronica
Copyright (c) 1992 Universidad de Sevilla

Permission to use, copy, modify, distribute, and sell this software and
its documentation for any purpose is hereby granted without fee,
provided that (i) the above copyright notices and this permission
notice appear in all copies of the software and related documentation,
and (ii) the names of Centro Nacional de Microelectronica and
Universidad de Sevilla may not be used in any advertising or publicity
relating to the software without the specific, prior written
permission.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL THE AUTHOR, CENTRO NACIONAL DE MICROELECTRONICA OR
UNIVERSIDAD DE SEVILLA BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT
OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

************************************************************************/
#include <stdio.h>
#include <X11/Xlib.h>
#include "xbdfdump.h"

#ifndef FORMAT_FILE
#define FORMAT_FILE "/usr/lib/X11/fonts/properties.fmt"
#endif

Display        *theDisplay;
Screen         *theScreen;
GC              theGC;
Window          theRootWindow;
Pixmap          thePixmap;
XFontStruct    *theFontStruct;
Font            theFont;
char           *theFontName;
FILE           *theFile;
char           *theFileName;
Atom            Xa_Resolution_X;
Atom            Xa_Resolution_Y;
Atom            Xa_Pixel_Size;
unsigned        pixmapWidth;
unsigned        pixmapHeight;
unsigned        pixmapDepth;
unsigned        fontWidth;
unsigned        fontHeight;
unsigned        fontMin1;
unsigned        fontMax1;
unsigned        fontMin2;
unsigned        fontMax2;
Bool            fontIsTwoByte;
unsigned        fontResolutionY;
unsigned        fontPointSize;
unsigned        charsInFont;

extern char    *Version;

static void     usage();

main(argc, argv)
int             argc;
char          **argv;
{
    char           *displayName = (char *) NULL;
    char            line[MAXLINELEN];
    char            filename[MAXFILENAME];
    FILE           *fontDir;
    XPixmapFormatValues *formats;
    int             i;
    int             nformats;
    int             fontcnt;
    char           *formatFile;
    char           *progName;

    formatFile = FORMAT_FILE;
    progName = argv[0];
    argv++;
    argc--;
    while (argv[0] && (argv[0])[0] == '-')
    {
	if (!strcmp(argv[0], "-d") || !strcmp(argv[0], "-display"))
	{
	    displayName = argv[1];
	    argc -= 2;
	    argv += 2;
	}
	else if (!strcmp(argv[0], "-f") || !strcmp(argv[0], "-fmt") ||
		 !strcmp(argv[0], "-format"))
	{
	    formatFile = argv[1];
	    argc -= 2;
	    argv += 2;
	}
	else if (!strcmp(argv[0], "-h") || !strcmp(argv[0], "-help"))
	{
	    usage(progName);
	    exit(1);
	}
	else if (!strcmp(argv[0], "-v"))
	{
	    fprintf(stderr, "%s\n", Version);
	    exit(0);
	}
	else
	{
	    break;
	}
    }

    if (!read_property_formats(formatFile))
    {
	exit(1);
    }

    if ((theDisplay = XOpenDisplay(displayName)) == (Display *) NULL)
    {
	fprintf(stderr, "Can't open display\n");
	exit(1);
    }

    theScreen = DefaultScreenOfDisplay(theDisplay);

    theRootWindow = DefaultRootWindow(theDisplay);

    /* Get Atoms that we are going to use. */
    Xa_Resolution_X = XInternAtom(theDisplay, "RESOLUTION_X", False);
    Xa_Resolution_Y = XInternAtom(theDisplay, "RESOLUTION_Y", False);
    Xa_Pixel_Size = XInternAtom(theDisplay, "PIXEL_SIZE", False);

    /* Find the minimum depth for the pixmap. */
    if (!(formats = XListPixmapFormats(theDisplay, &nformats)))
    {
	fprintf(stderr, "Can't get pixmap formats.\n");
	exit(1);
    }
    pixmapDepth = formats[0].depth;
    for (i = 1; i < nformats; i++)
    {
	pixmapDepth = formats[i].depth < pixmapDepth ?
	    formats[i].depth : pixmapDepth;
    }
    XFree(formats);
    if (argc > 0)
    {
	while (argc > 0)
	{
	    xbdfdump(argv[0], argv[1]);
	    argc -= 2;
	}
    }
    else
    {
	if (!(fontDir = fopen("fonts.dir", "r")))
	{
	    fprintf(stderr, "Can't open fonts.dir\n");
	    exit(1);
	}
	fgets(line, MAXLINELEN, fontDir);
	if (sscanf(line, "%d", &fontcnt) != 1)
	{
	    fprintf(stderr, "Bad format in fonts.dir.\n");
	    exit(1);
	}
	for (i = 0; i < fontcnt; i++)
	{
	    char           *cptr;
	    int             l;

	    if (!fgets(line, MAXLINELEN, fontDir))
	    {
		fprintf(stderr, "Too few fonts in fonts.dir.\n");
		exit(1);
	    }

	    /* Get the file name and change extension to bdf. */
	    sscanf(line, "%s", filename);
	    l = strlen(filename) - 1;
	    while (l > 0 && filename[l] != '.')
	    {
		l--;
	    }
	    if (l)
	    {
		strcpy(filename + l, ".bdf");
	    }
	    else
	    {
		strcat(filename, ".bdf");
	    }

	    /* Get the font name. */
	    cptr = line - 1;
	    while (!isspace(*++cptr));	/* Skip the file. */
	    while (isspace(*++cptr));	/* Skip spaces. */
	    l = strlen(cptr);
	    cptr[l - 1] = '\0';	/* Remove newline. */
	    xbdfdump(cptr, filename);
	}
    }
    exit(0);
}

static void     usage(prog)
char           *prog;
{
    fprintf(stderr,
	"Usage: %s [-display name ] [-format file] [font [file [font...]\n",
	    prog);
    fprintf(stderr, "-display: Specifies the server to connect.\n");
    fprintf(stderr, "-format:  Specifies the properties format file.\n");
}
