
static char    *SccsId = "@(#)properties.c 1.1 5/25/92 CNM-US; C source";

/************************************************************************

Copyright (c) 1992 Juan D. Martin
Copyright (c) 1992 Centro Nacional de Microelectronica
Copyright (c) 1992 Universidad de Sevilla

Permission to use, copy, modify, distribute, and sell this software and
its documentation for any purpose is hereby granted without fee,
provided that (i) the above copyright notices and this permission
notice appear in all copies of the software and related documentation,
and (ii) the names of Centro Nacional de Microelectronica and
Universidad de Sevilla may not be used in any advertising or publicity
relating to the software without the specific, prior written
permission.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL THE AUTHOR, CENTRO NACIONAL DE MICROELECTRONICA OR
UNIVERSIDAD DE SEVILLA BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT
OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include "xbdfdump.h"

extern char    *malloc();

struct property_fmt_s
{
    char           *name;
    char           *format;
    Bool            is_string;
    struct property_fmt_s *next;
};

typedef struct property_fmt_s property_fmt;

static property_fmt *formats;

Bool            read_property_formats(file)
char           *file;
{
    FILE           *fh;
    char            line[MAXLINELEN];
    char            tmpfmt[MAXLINELEN];
    char           *cptr;
    char           *fptr;
    char           *prop;
    char           *fmt;
    Bool            isString;
    int             lcnt = 0;
    int             llen;
    property_fmt   *last_fmt = (property_fmt *) 0;

    if (!(fh = fopen(file, "r")))
    {
	fprintf(stderr, "Can't open property format file %s\n", file);
	return False;
    }
    while (fgets(line, MAXLINELEN, fh))
    {
	lcnt++;
	llen = strlen(line);
	isString = False;
	if (llen >= MAXLINELEN - 1)
	{
	    fprintf(stderr, "Line %d too long in %s\n", lcnt, file);
	    fclose(fh);
	    return False;
	}
	if (line[0] == '#')
	{
	    continue;
	}
	cptr = line - 1;
	/* Skip initial spaces. */
	while (isspace(*++cptr));
	/* Property name begins here. */
	prop = cptr;
	while (!isspace(*++cptr));
	if (*cptr == '\n')
	{
	    fprintf(stderr, "Bad format in line %d, file %s\n", lcnt, file);
	    continue;
	}
	*cptr = '\0';
	/* Skip separation spaces. */
	while (isspace(*++cptr));
	/* Must be the initial quote. */
	if (*cptr++ != '"')
	{
	    fprintf(stderr, "Bad format in line %d, file %s\n", lcnt, file);
	    continue;
	}
	fptr = tmpfmt;
	while (*cptr)
	{
	    switch (*cptr)
	    {
	    case '\n':
		fprintf(stderr, "Bad format in line %d, file %s\n", lcnt, file);
		fmt = (char *) 0;
		*cptr = '\0';
		break;
	    case '"':
		/* Format ends here. */
		fmt = tmpfmt;
		*fptr = '\0';
		*cptr = '\0';
		break;
	    case '\\':
		cptr++;
		/* Handle escapes. */
		switch (*cptr)
		{
		case 't':
		    *fptr++ = '\t';
		    break;
		case '\n':
		    fprintf(stderr, "Bad format in line %d, file %s\n",
			    lcnt, file);
		    fmt = (char *) 0;
		    *cptr = '\0';
		    break;
		default:
		    *fptr++ = *cptr++;
		}
	    case '%':
		/* Find out string cases. */
		if (cptr[1] == 's')
		{
		    isString = True;
		}
	    default:
		*fptr++ = *cptr++;
	    }
	}
	/* Skip bad lines. */
	if (!fmt)
	{
	    continue;
	}
	/* Append to the list. */
	if (!last_fmt)
	{
	    formats = (property_fmt *) malloc(sizeof(property_fmt));
	    last_fmt = formats;
	}
	else
	{
	    last_fmt->next =
		(property_fmt *) malloc(sizeof(property_fmt));
	    last_fmt = last_fmt->next;
	}
	last_fmt->name = strdup(prop);
	last_fmt->format = strdup(fmt);
	last_fmt->is_string = isString;
	last_fmt->next = (property_fmt *) 0;
    }
    fclose(fh);
    return True;
}

/* Convert a number in a property into a string suitable for BDF. */
static char    *data_to_string(v)
unsigned long   v;
{
    char           *s;
    char           *r;
    char           *sptr;
    char           *rptr;

    s = XGetAtomName(theDisplay, (Atom) v);
    if (!s)
    {
	return (char *) 0;
    }
    r = malloc(strlen(s) * 2 + 3);
    sptr = s;
    rptr = r;
    *rptr++ = '"';
    while (*sptr)
    {
	switch (*sptr)
	{
	case '"':
	    /* Duplicate quotes. */
	    *rptr++ = '"';
	    *rptr++ = '"';
	    sptr++;
	    break;
	case '\n':
	    /* Convert newlines into blanks. */
	    *rptr++ = ' ';
	    sptr++;
	    break;
	default:
	    *rptr++ = *sptr++;
	}
    }
    *rptr++ = '"';
    *rptr++ = '\0';
    XFree(s);
    return r;
}

static char    *unknown = "UNKNOWN_PROPERTY";

void            print_property(name, value)
Atom            name;
unsigned long   value;
{
    char           *sname;
    char           *svalue = (char *) 0;
    property_fmt   *fmt;

    sname = XGetAtomName(theDisplay, name);
    if (!sname)
    {
	/* This should never happen... */
	fprintf(stderr, "Warning: font %s , file %s: Bad property atom %u\n",
		theFontName, theFileName, (unsigned long) name);
	sname = unknown;
    }
    fmt = formats;
    /* Find its format. */
    while (fmt)
    {
	if (!strcmp(fmt->name, sname))
	{
	    break;
	}
	fmt = fmt->next;
    }
    fputs(sname, theFile);
    putc(' ', theFile);
    if (!fmt)
    {
	fprintf(stderr, "Warning: font %s , file %s: guessing %s format\n",
		theFontName, theFileName, sname);
	if (value > XA_LAST_PREDEFINED && (svalue = data_to_string(value)))
	{
	    fprintf(theFile, "%s", svalue);
	}
	else
	{
	    fprintf(theFile, "%ld", value);
	}
    }
    else
    {
	if (fmt->is_string)
	{
	    if (!(svalue = data_to_string(value)))
	    {
		fprintf(stderr,
		"Warning: font %s , file %s: string data not found for %s.\n",
			theFontName, theFileName, sname);
		svalue = "";
	    }
	    fprintf(theFile, fmt->format, svalue);
	}
	else
	{
	    fprintf(theFile, fmt->format, value);
	}
    }
    putc('\n', theFile);
    if (svalue)
    {
	free(svalue);
    }
    if (sname != unknown)
    {
	XFree(sname);
    }
}
