
static char    *SccsId = "@(#)xbdfdump.c 1.1 5/25/92 CNM-US; C source";

/************************************************************************

Copyright (c) 1992 Juan D. Martin
Copyright (c) 1992 Centro Nacional de Microelectronica
Copyright (c) 1992 Universidad de Sevilla

Permission to use, copy, modify, distribute, and sell this software and
its documentation for any purpose is hereby granted without fee,
provided that (i) the above copyright notices and this permission
notice appear in all copies of the software and related documentation,
and (ii) the names of Centro Nacional de Microelectronica and
Universidad de Sevilla may not be used in any advertising or publicity
relating to the software without the specific, prior written
permission.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

IN NO EVENT SHALL THE AUTHOR, CENTRO NACIONAL DE MICROELECTRONICA OR
UNIVERSIDAD DE SEVILLA BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT
OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

************************************************************************/
#include <stdio.h>
#include <X11/Xlib.h>
#include "xbdfdump.h"

#ifndef NCHARS_X
#define NCHARS_X 5
#endif

#ifndef NCHARS_Y
#define NCHARS_Y 3
#endif

void            xbdfdump(font, file)
char           *font;
char           *file;
{
    XGCValues       gcValues;
    XCharStruct    *chars;
    XImage         *ximage;
    unsigned        cnt1 = 0;
    unsigned        cnt2;
    unsigned        posX;
    unsigned        posY;
    int             charLbearing;
    int             charRbearing;
    int             charWidth;
    int             charAscent;
    int             charDescent;
    int             charAttributes;
    char           *charName;
    unsigned        charEncoding;

    theFontName = font;
    theFontStruct = XLoadQueryFont(theDisplay, theFontName);
    if (!theFontStruct)
    {
	fprintf(stderr, "%s not found\n", font);
	return;
    }
    if (!file)
    {
	theFileName = "(stdout)";
	theFile = stdout;
    }
    else
    {
	theFileName = file;
	if (!(theFile = fopen(file, "w+")))
	{
	    fprintf(stderr, "Can't open %s\n", file);
	    return;
	}
    }
    chars = theFontStruct->per_char;
    theFont = theFontStruct->fid;
    fontMin1 = theFontStruct->min_byte1;
    fontMax1 = theFontStruct->max_byte1;
    fontMin2 = theFontStruct->min_char_or_byte2;
    fontMax2 = theFontStruct->max_char_or_byte2;
    fontIsTwoByte = fontMax1 ? True : False;
    print_header();
    pixmapWidth = fontWidth * NCHARS_X;
    pixmapHeight = fontHeight * NCHARS_Y;
    thePixmap = XCreatePixmap(theDisplay, theRootWindow,
			      pixmapWidth, pixmapHeight,
			      pixmapDepth);
    gcValues.font = theFont;
    theGC = XCreateGC(theDisplay, (Drawable) thePixmap, GCFont, &gcValues);
    /* If there is no per_char structure, use this for all characters. */
    if (!chars)
    {
	charLbearing = theFontStruct->max_bounds.lbearing;
	charRbearing = theFontStruct->max_bounds.rbearing;
	charWidth = theFontStruct->max_bounds.width;
	charAscent = theFontStruct->max_bounds.ascent;
	charDescent = theFontStruct->max_bounds.descent;
	charAttributes = theFontStruct->max_bounds.attributes;
    }
    cnt2 = 0;
    while (cnt1 < charsInFont)
    {
	/* Clear the pixmap */
	XSetForeground(theDisplay, theGC, 0);
	XFillRectangle(theDisplay, (Drawable) thePixmap, theGC,
		       0, 0, pixmapWidth + 1, pixmapHeight + 1);
	XSetForeground(theDisplay, theGC, 1);

	/* First loop: Draw as many chars as would fit in the pixmap. */
	posX = 0;
	posY = 0;
	while (cnt2 < charsInFont)
	{
	    if (chars)
	    {
		charLbearing = chars[cnt2].lbearing;
		charRbearing = chars[cnt2].rbearing;
		{
		    /* No need to draw this one. */
		    if (charLbearing == charRbearing)
		    {
			cnt2++;
			continue;
		    }
		}
		charAscent = chars[cnt2].ascent;
	    }
	    draw_char(cnt2, posX, posY, charLbearing, charAscent);
	    cnt2++;
	    posX += fontWidth;
	    if (posX >= pixmapWidth)
	    {
		posY += fontHeight;
		if (posY >= pixmapHeight)
		{
		    break;
		}
		posX = 0;
	    }
	}
	ximage = XGetImage(theDisplay, (Drawable) thePixmap,
			   0, 0, pixmapWidth, pixmapHeight,
			   1, XYPixmap);

	/* Second loop: Scan the characters drawn and do output. */
	cnt2 = cnt1;
	posX = 0;
	posY = 0;
	while (cnt2 < charsInFont)
	{
	    /* Get character metrics if per_char structure. */
	    if (chars)
	    {
		charLbearing = chars[cnt2].lbearing;
		charRbearing = chars[cnt2].rbearing;
		charWidth = chars[cnt2].width;
		charAscent = chars[cnt2].ascent;
		charDescent = chars[cnt2].descent;
		charAttributes = chars[cnt2].attributes;
		{
		    if (charLbearing == charRbearing && charWidth == 0)
		    {
			cnt2++;
			continue;
		    }
		}
	    }
	    charEncoding = index_to_encoding(cnt2);
	    fputs("STARTCHAR ", theFile);
	    if (charName = XKeysymToString((KeySym) charEncoding))
	    {
		fputs(charName, theFile);
	    }
	    else
	    {
		fprintf(theFile,
			fontIsTwoByte ? "C%06o" : "C%03o", charEncoding);
	    }
	    putc('\n', theFile);
	    fprintf(theFile, "ENCODING %d\n", charEncoding);
	    fprintf(theFile, "SWIDTH %ld 0\n",
		    (long) (charWidth * 72000L) /
		    (fontPointSize * fontResolutionY));
	    fprintf(theFile, "DWIDTH %d 0\n", charWidth);
	    fprintf(theFile, "BBX %u %u %d %d\n",
		    charRbearing - charLbearing,
		    charAscent + charDescent,
		    charLbearing,
		    -charDescent);
	    if (charAttributes)
	    {
		fprintf(theFile, "ATTRIBUTES %04x\n", charAttributes);
	    }
	    fputs("BITMAP\n", theFile);
	    /* This one was not drawn, but we may still have a zeroed pixmap. */
	    if (charLbearing == charRbearing)
	    {
		int             i,
		                j;

		for (i = 0; i < charAscent + charDescent; i++)
		{
		    fputs("00\n", theFile);
		}
		fputs("ENDCHAR\n", theFile);
		cnt2++;
		continue;
	    }
	    scan_char(ximage, posX, posY, charRbearing - charLbearing,
		      charAscent + charDescent);
	    fputs("ENDCHAR\n", theFile);
	    cnt2++;
	    posX += fontWidth;
	    if (posX >= pixmapWidth)
	    {
		posY += fontHeight;
		if (posY >= pixmapHeight)
		{
		    break;
		}
		posX = 0;
	    }
	}
	cnt1 = cnt2;
	XDestroyImage(ximage);
    }

    fputs("ENDFONT\n", theFile);

    /* Release everything. */
    fclose(theFile);
    XFreeGC(theDisplay, theGC);
    XFreePixmap(theDisplay, thePixmap);
    XFreeFont(theDisplay, theFontStruct);
}
